@@ -4,24 +4,44 @@
 # license information.
 # --------------------------------------------------------------------------
 """Uses the Azure Python SDK to collect and return details related to Azure."""
-from typing import Optional, Dict, Tuple
+from typing import Optional, Dict, Tuple, List
 import datetime
 
 import attr
 import pandas as pd
 import numpy as np
 
-from azure.common.credentials import ServicePrincipalCredentials
 from azure.mgmt.subscription import SubscriptionClient
-from azure.mgmt.resource import ResourceManagementClient
-from azure.mgmt.network import NetworkManagementClient
-from azure.mgmt.monitor import MonitorManagementClient
-from azure.mgmt.compute import ComputeManagementClient
-from azure.mgmt.compute.models import VirtualMachineInstanceView
 from azure.common.exceptions import CloudError
 
-from ..common.provider_settings import get_provider_settings
-from ..common.utility import MsticpyException
+from ..common.azure_auth import az_connect
+from ..common.azure_auth_core import AzCredentials
+
+from ..common.exceptions import (
+    MsticpyAzureConfigError,
+    MsticpyNotConnectedError,
+    MsticpyResourceException,
+    MsticpyImportExtraError,
+)
+
+try:
+    from azure.mgmt.resource import ResourceManagementClient
+    from azure.mgmt.network import NetworkManagementClient
+
+    try:
+        # Try new version but keep backward compat with 1.0.1
+        from azure.mgmt.monitor import MonitorManagementClient
+    except ImportError:
+        from azure.mgmt.monitor import MonitorClient as MonitorManagementClient
+    from azure.mgmt.compute import ComputeManagementClient
+    from azure.mgmt.compute.models import VirtualMachineInstanceView
+except ImportError as imp_err:
+    raise MsticpyImportExtraError(
+        "Cannot use this feature without azure packages installed",
+        title="Error importing azure module",
+        extra="azure",
+    ) from imp_err
+
 from .._version import VERSION
 
 __version__ = VERSION
@@ -86,51 +106,32 @@ class InterfaceItems:
     subnet_route_table = attr.ib()
 
 
-class MsticpyAzureException(MsticpyException):
-    """Exception class for AzureData."""
-
-
-# pylint: enable=too-few-public-methods, too-many-instance-attributes
-
-
 class AzureData:
     """Class for returning data on an Azure tenant."""
 
     def __init__(self, connect: bool = False):
         """Initialize connector for Azure Python SDK."""
         self.connected = False
-        self.credentials: Optional[ServicePrincipalCredentials] = None
+        self.credentials: Optional[AzCredentials] = None
         self.sub_client: Optional[SubscriptionClient] = None
         self.resource_client: Optional[ResourceManagementClient] = None
         self.network_client: Optional[NetworkManagementClient] = None
         self.monitoring_client: Optional[MonitorManagementClient] = None
         self.compute_client: Optional[ComputeManagementClient] = None
-        if connect is True:
+        if connect:
             self.connect()
 
-    def connect(self, client_id: str = None, tenant_id: str = None, secret: str = None):
+    def connect(
+        self,
+        auth_methods: List = None,
+        silent: bool = False,
+    ):
         """Authenticate with the SDK."""
-        # Use details of msticpyyaml if not provided
-        if client_id is None and tenant_id is None and secret is None:
-            data_provs = get_provider_settings(config_section="DataProviders")
-            az_cli_config = data_provs.get("AzureCLI")
-            # az_cli_config = config.settings.get("AzureCLI")
-            if not az_cli_config:
-                raise MsticpyAzureException(
-                    "No AzureCLI configuration found in configuration settings."
-                )
-            config_items = az_cli_config.args
-            client_id = config_items["clientId"]
-            tenant_id = config_items["tenantId"]
-            secret = config_items["clientSecret"]
-
-        # Create credentials and connect to the subscription client to validate
-        self.credentials = ServicePrincipalCredentials(
-            client_id=client_id, secret=secret, tenant=tenant_id
-        )
+        self.credentials = az_connect(auth_methods=auth_methods, silent=silent)
         if not self.credentials:
             raise CloudError("Could not obtain credentials.")
-        self.sub_client = SubscriptionClient(self.credentials)
+        self._check_client("sub_client")
+        self.sub_client = SubscriptionClient(self.credentials.legacy)
         if not self.sub_client:
             raise CloudError("Could not create a Subscription client.")
         self.connected = True
@@ -138,17 +139,32 @@ class AzureData:
     def get_subscriptions(self) -> pd.DataFrame:
         """Get details of all subscriptions within the tenant."""
         if self.connected is False:
-            raise MsticpyAzureException("Please connect before continuing")
+            raise MsticpyNotConnectedError(
+                "You need to connect to the service before using this function.",
+                help_uri=MsticpyAzureConfigError.DEF_HELP_URI,
+                title="Please call connect() before continuing.",
+            )
 
         subscription_ids = []
         display_names = []
         states = []
-        for item in self.sub_client.subscriptions.list():  # type: ignore
+        # pylint: disable=unnecessary-comprehension
+        try:
+            sub_list = [
+                sub for sub in self.sub_client.subscriptions.list()  # type: ignore
+            ]
+        except AttributeError:
+            self._legacy_auth("sub_client")
+            sub_list = [
+                sub for sub in self.sub_client.subscriptions.list()  # type: ignore
+            ]
+
+        for item in sub_list:  # type: ignore
             subscription_ids.append(item.subscription_id)
             display_names.append(item.display_name)
             states.append(str(item.state))
 
-        sub_details = pd.DataFrame(
+        return pd.DataFrame(
             {
                 "Subscription ID": subscription_ids,
                 "Display Name": display_names,
@@ -156,8 +172,6 @@ class AzureData:
             }
         )
 
-        return sub_details
-
     def get_subscription_info(self, sub_id: str) -> dict:
         """
         Get information on a specific subscription.
@@ -169,10 +183,18 @@ class AzureData:
 
         """
         if self.connected is False:
-            raise MsticpyAzureException("Please connect before continuing")
+            raise MsticpyNotConnectedError(
+                "You need to connect to the service before using this function.",
+                help_uri=MsticpyAzureConfigError.DEF_HELP_URI,
+                title="Please call connect() before continuing.",
+            )
+        try:
+            sub = self.sub_client.subscriptions.get(sub_id)  # type: ignore
+        except AttributeError:
+            self._legacy_auth("sub_client")
+            sub = self.sub_client.subscriptions.get(sub_id)  # type: ignore
 
-        sub = self.sub_client.subscriptions.get(sub_id)  # type: ignore
-        sub_details = {
+        return {
             "Subscription ID": sub.subscription_id,
             "Display Name": sub.display_name,
             "State": str(sub.state),
@@ -181,9 +203,7 @@ class AzureData:
             "Spending Limit": sub.subscription_policies.spending_limit,
         }
 
-        return sub_details
-
-    def get_resources(
+    def get_resources(  # noqa: MC0001
         self, sub_id: str, rgroup: str = None, get_props: bool = False
     ) -> pd.DataFrame:
         """
@@ -207,26 +227,33 @@ class AzureData:
         """
         # Check if connection and client required are already present
         if self.connected is False:
-            raise MsticpyAzureException("Please connect before continuing")
+            raise MsticpyNotConnectedError(
+                "You need to connect to the service before using this function.",
+                help_uri=MsticpyAzureConfigError.DEF_HELP_URI,
+                title="Please call connect() before continuing.",
+            )
 
         self._check_client("resource_client", sub_id)
 
+        resources = []  # type: List
         if rgroup is None:
-            resources = self.resource_client.resources.list()  # type: ignore
+            for resource in self.resource_client.resources.list():  # type: ignore
+                resources.append(resource)
         else:
-            resources = self.resource_client.resources.list_by_resource_group(  # type: ignore
+            for resource in self.resource_client.resources.list_by_resource_group(  # type: ignore
                 rgroup
-            )
+            ):
+                resources.append(resource)
 
         # Warn users about getting full properties for each resource
-        if get_props is True:
+        if get_props:
             print("Collecting properties for every resource may take some time...")
 
         resource_items = []
 
         # Get properites for each resource
         for resource in resources:
-            if get_props is True:
+            if get_props:
                 if resource.type == "Microsoft.Compute/virtualMachines":
                     state = self._get_compute_state(
                         resource_id=resource.id, sub_id=sub_id
@@ -240,7 +267,7 @@ class AzureData:
 
                 except CloudError:
                     props = self.resource_client.resources.get_by_id(  # type: ignore
-                        resource.id, self._get_api(resource.id)
+                        resource.id, self._get_api(resource.id, sub_id=sub_id)
                     ).properties
             else:
                 props = resource.properties
@@ -265,11 +292,9 @@ class AzureData:
             )
             resource_items.append(resource_details)
 
-        resource_df = pd.DataFrame(resource_items)
+        return pd.DataFrame(resource_items)
 
-        return resource_df
-
-    def get_resource_details(
+    def get_resource_details(  # noqa: MC0001
         self, sub_id: str, resource_id: str = None, resource_details: dict = None
     ) -> dict:
         """
@@ -297,35 +322,63 @@ class AzureData:
         """
         # Check if connection and client required are already present
         if self.connected is False:
-            raise MsticpyAzureException("Please connect before continuing")
-
+            raise MsticpyNotConnectedError(
+                "You need to connect to the service before using this function.",
+                help_uri=MsticpyAzureConfigError.DEF_HELP_URI,
+                title="Please call connect() before continuing.",
+            )
         self._check_client("resource_client", sub_id)
 
         # If a resource id is provided use get_by_id to get details
         if resource_id is not None:
-            resource = self.resource_client.resources.get_by_id(  # type: ignore
-                resource_id, api_version=self._get_api(resource_id)
-            )
+            try:
+                resource = self.resource_client.resources.get_by_id(  # type: ignore
+                    resource_id, api_version=self._get_api(resource_id, sub_id=sub_id)
+                )
+            except AttributeError:
+                self._legacy_auth("resource_client", sub_id)
+                resource = self.resource_client.resources.get_by_id(  # type: ignore
+                    resource_id, api_version=self._get_api(resource_id, sub_id=sub_id)
+                )
             if resource.type == "Microsoft.Compute/virtualMachines":
                 state = self._get_compute_state(resource_id=resource_id, sub_id=sub_id)
             else:
                 state = None
         # If resource details are provided use get to get details
         elif resource_details is not None:
-            resource = self.resource_client.resources.get(  # type: ignore
-                resource_details["resource_group_name"],
-                resource_details["resource_provider_namespace"],
-                resource_details["parent_resource_path"],
-                resource_details["resource_type"],
-                resource_details["resource_name"],
-                api_version=self._get_api(
-                    resource_provider=(
-                        resource_details["resource_provider_namespace"]
-                        + "/"
-                        + resource_details["resource_type"]
-                    )
-                ),
-            )
+            try:
+                resource = self.resource_client.resources.get(  # type: ignore
+                    resource_details["resource_group_name"],
+                    resource_details["resource_provider_namespace"],
+                    resource_details["parent_resource_path"],
+                    resource_details["resource_type"],
+                    resource_details["resource_name"],
+                    api_version=self._get_api(
+                        resource_provider=(
+                            resource_details["resource_provider_namespace"]
+                            + "/"
+                            + resource_details["resource_type"]
+                        ),
+                        sub_id=sub_id,
+                    ),
+                )
+            except AttributeError:
+                self._legacy_auth("resource_client", sub_id)
+                resource = self.resource_client.resources.get(  # type: ignore
+                    resource_details["resource_group_name"],
+                    resource_details["resource_provider_namespace"],
+                    resource_details["parent_resource_path"],
+                    resource_details["resource_type"],
+                    resource_details["resource_name"],
+                    api_version=self._get_api(
+                        resource_provider=(
+                            resource_details["resource_provider_namespace"]
+                            + "/"
+                            + resource_details["resource_type"]
+                        ),
+                        sub_id=sub_id,
+                    ),
+                )
             state = None
         else:
             raise ValueError("Please provide either a resource ID or resource details")
@@ -350,7 +403,7 @@ class AzureData:
 
         return resource_details
 
-    def _get_api(
+    def _get_api(  # noqa: MC0001
         self, resource_id: str = None, sub_id: str = None, resource_provider: str = None
     ) -> str:
         """
@@ -373,7 +426,11 @@ class AzureData:
         """
         # Check if connection and client required are already present
         if self.connected is False:
-            raise MsticpyAzureException("Please connect before continuing")
+            raise MsticpyNotConnectedError(
+                "You need to connect to the service before using this function.",
+                help_uri=MsticpyAzureConfigError.DEF_HELP_URI,
+                title="Please call connect() before continuing.",
+            )
 
         self._check_client("resource_client", sub_id)  # type: ignore
 
@@ -382,46 +439,50 @@ class AzureData:
             try:
                 namespace = resource_id.split("/")[6]
                 service = resource_id.split("/")[7]
-            except IndexError:
-                raise MsticpyAzureException(
+            except IndexError as idx_err:
+                raise MsticpyResourceException(
                     "Provided Resource ID isn't in the correct format.",
                     "It should look like:",
                     "/subscriptions/SUB_ID/resourceGroups/RESOURCE_GROUP/"
                     + "providers/NAMESPACE/SERVICE_NAME/RESOURCE_NAME ",
-                )
+                ) from idx_err
 
         elif resource_provider is not None:
             try:
                 namespace = resource_provider.split("/")[0]
                 service = resource_provider.split("/")[1]
-            except IndexError:
-                raise MsticpyAzureException(
-                    """Provided Resource Provider isn't in the correct format. It should look like:
-                       NAMESPACE/SERVICE_NAME"""
-                )
+            except IndexError as idx_err:
+                raise MsticpyResourceException(
+                    "Provided Resource Provider isn't in the correct format.",
+                    "It should look like: NAMESPACE/SERVICE_NAME",
+                ) from idx_err
         else:
             raise ValueError(
                 "Please provide an resource ID or resource provider namespace"
             )
 
         # Get list of API versions for the service
-        provider = self.resource_client.providers.get(namespace)  # type: ignore
+        try:
+            provider = self.resource_client.providers.get(namespace)  # type: ignore
+        except AttributeError:
+            self._legacy_auth("resource_client", sub_id)
+            provider = self.resource_client.providers.get(namespace)  # type: ignore
+
         resource_types = next(
             (t for t in provider.resource_types if t.resource_type == service), None
         )
 
         # Get first API version that isn't in preview
-        if resource_types:
-            api_version = [
-                v for v in resource_types.api_versions if "preview" not in v.lower()
-            ]
-            if api_version is None or not api_version:
-                api_ver = resource_types.api_versions[0]
-            else:
-                api_ver = api_version[0]
-        else:
-            raise MsticpyAzureException("Resource provider not found")
+        if not resource_types:
+            raise MsticpyResourceException("Resource provider not found")
 
+        api_version = [
+            v for v in resource_types.api_versions if "preview" not in v.lower()
+        ]
+        if api_version is None or not api_version:
+            api_ver = resource_types.api_versions[0]
+        else:
+            api_ver = api_version[0]
         return str(api_ver)
 
     def get_network_details(
@@ -445,14 +506,25 @@ class AzureData:
         """
         # Check if connection and client required are already present
         if self.connected is False:
-            raise MsticpyAzureException("Please connect before continuing")
+            raise MsticpyNotConnectedError(
+                "You need to connect to the service before using this function.",
+                help_uri=MsticpyAzureConfigError.DEF_HELP_URI,
+                title="Please call connect() before continuing.",
+            )
 
         self._check_client("network_client", sub_id)
 
         # Get interface details and parse relevent elements into a dataframe
-        details = self.network_client.network_interfaces.get(  # type: ignore
-            network_id.split("/")[4], network_id.split("/")[8]
-        )
+        try:
+            details = self.network_client.network_interfaces.get(  # type: ignore
+                network_id.split("/")[4], network_id.split("/")[8]
+            )
+        except AttributeError:
+            self._legacy_auth("network_client", sub_id)
+            details = self.network_client.network_interfaces.get(  # type: ignore
+                network_id.split("/")[4], network_id.split("/")[8]
+            )
+
         ips = []
         for ip in details.ip_configurations:  # pylint: disable=invalid-name
             ip_details = attr.asdict(
@@ -472,29 +544,31 @@ class AzureData:
 
         ip_df = pd.DataFrame(ips)
 
-        # Get NSG details and parse relevent elements into a dataframe
-        nsg_details = self.network_client.network_security_groups.get(  # type: ignore
-            details.network_security_group.id.split("/")[4],
-            details.network_security_group.id.split("/")[8],
-        )
-        nsg_rules = []
-        for nsg in nsg_details.default_security_rules:
-            rules = attr.asdict(
-                NsgItems(
-                    nsg.name,
-                    nsg.description,
-                    nsg.protocol,
-                    nsg.direction,
-                    nsg.source_port_range,
-                    nsg.destination_port_range,
-                    nsg.source_address_prefix,
-                    nsg.destination_address_prefix,
-                    nsg.access,
-                )
+        nsg_df = pd.DataFrame()
+        if details.network_security_group is not None:
+            # Get NSG details and parse relevent elements into a dataframe
+            nsg_details = self.network_client.network_security_groups.get(  # type: ignore
+                details.network_security_group.id.split("/")[4],
+                details.network_security_group.id.split("/")[8],
             )
-            nsg_rules.append(rules)
+            nsg_rules = []
+            for nsg in nsg_details.default_security_rules:
+                rules = attr.asdict(
+                    NsgItems(
+                        nsg.name,
+                        nsg.description,
+                        nsg.protocol,
+                        nsg.direction,
+                        nsg.source_port_range,
+                        nsg.destination_port_range,
+                        nsg.source_address_prefix,
+                        nsg.destination_address_prefix,
+                        nsg.access,
+                    )
+                )
+                nsg_rules.append(rules)
 
-        nsg_df = pd.DataFrame(nsg_rules)
+            nsg_df = pd.DataFrame(nsg_rules)
 
         return ip_df, nsg_df
 
@@ -536,13 +610,17 @@ class AzureData:
         elif sample_time.casefold().startswith("m"):
             interval = "PT1M"
         else:
-            raise MsticpyAzureException(
-                "Select how often you want to sample data - 'hour', or 'minute'"
+            raise ValueError(
+                "invalid value for sample_time - specify 'hour', or 'minute'"
             )
 
         # Check if connection and client required are already present
         if self.connected is False:
-            raise MsticpyAzureException("Please connect before continuing")
+            raise MsticpyNotConnectedError(
+                "You need to connect to the service before using this function.",
+                help_uri=MsticpyAzureConfigError.DEF_HELP_URI,
+                title="Please call connect() before continuing.",
+            )
 
         self._check_client("monitoring_client", sub_id)
 
@@ -550,14 +628,23 @@ class AzureData:
         start = datetime.datetime.now().date()
         end = start - datetime.timedelta(days=start_time)
 
-        mon_details = self.monitoring_client.metrics.list(  # type: ignore
-            resource_id,
-            timespan=f"{end}/{start}",
-            interval=interval,
-            metricnames=f"{metrics}",
-            aggregation="Total",
-        )
-
+        try:
+            mon_details = self.monitoring_client.metrics.list(  # type: ignore
+                resource_id,
+                timespan=f"{end}/{start}",
+                interval=interval,
+                metricnames=f"{metrics}",
+                aggregation="Total",
+            )
+        except AttributeError:
+            self._legacy_auth("monitoring_client", sub_id)
+            mon_details = self.monitoring_client.metrics.list(  # type: ignore
+                resource_id,
+                timespan=f"{end}/{start}",
+                interval=interval,
+                metricnames=f"{metrics}",
+                aggregation="Total",
+            )
         results = {}
         # Create a dict of all the results returned
         for metric in mon_details.value:
@@ -595,7 +682,11 @@ class AzureData:
 
         """
         if self.connected is False:
-            raise MsticpyAzureException("Please connect before continuing")
+            raise MsticpyNotConnectedError(
+                "You need to connect to the service before using this function.",
+                help_uri=MsticpyAzureConfigError.DEF_HELP_URI,
+                title="Please call connect() before continuing.",
+            )
 
         self._check_client("compute_client", sub_id)
 
@@ -605,26 +696,54 @@ class AzureData:
         name = r_details[r_details.index("virtualMachines") + 1]
 
         # Get VM instance details and return them
-        instance_details = self.compute_client.virtual_machines.instance_view(  # type: ignore
-            r_group, name
-        )
+        try:
+            instance_details = self.compute_client.virtual_machines.instance_view(  # type: ignore
+                r_group, name
+            )
+        except AttributeError:
+            self._legacy_auth("compute_client", sub_id)
+            instance_details = self.compute_client.virtual_machines.instance_view(  # type: ignore
+                r_group, name
+            )
+
         return instance_details
 
-    def _check_client(self, client_name: str, sub_id: str):
+    def _check_client(self, client_name: str, sub_id: str = None):
         """
         Check required client is present, if not create it.
 
         Parameters
         ----------
-        self:
-        client_name:
+        client_name : str
             The name of the client to be checked.
-        sub_id:
-            The subscription ID for the client to connect to.
+        sub_id : str, optional
+            The subscription ID for the client to connect to, by default None
+
+        """
+        if getattr(self, client_name) is None:
+            client = _CLIENT_MAPPING[client_name]
+            if sub_id is None:
+                setattr(self, client_name, client(self.credentials.modern))  # type: ignore
+            else:
+                setattr(self, client_name, client(self.credentials.modern, sub_id))  # type: ignore
+
+        if getattr(self, client_name) is None:
+            raise CloudError("Could not create client")
+
+    def _legacy_auth(self, client_name: str, sub_id: str = None):
+        """
+        Create client with v1 authentication token.
+
+        Parameters
+        ----------
+        client_name : str
+            The name of the client to be checked.
+        sub_id : str, optional
+            The subscription ID for the client to connect to, by default None
 
         """
         client = _CLIENT_MAPPING[client_name]
-        if getattr(self, client_name) is None:
-            setattr(self, client_name, client(self.credentials, sub_id))
-            if getattr(self, client_name) is None:
-                raise CloudError("Could not create client")
+        if sub_id is None:
+            setattr(self, client_name, client(self.credentials.legacy))  # type: ignore
+        else:
+            setattr(self, client_name, client(self.credentials.legacy, sub_id))  # type: ignore
