@@ -7,10 +7,11 @@
 from datetime import datetime
 from typing import Any, Union, Set, Dict, Tuple, List
 
-import numpy as np
 import pandas as pd
+from pandas.api.types import is_datetime64_any_dtype
 from pandas.errors import OutOfBoundsDatetime
 from bokeh.io import output_notebook, show
+from bokeh.models.annotations import LegendItem
 from bokeh.models import (
     ColumnDataSource,
     DatetimeTickFormatter,
@@ -258,6 +259,10 @@ def display_timeline_values(
     """
     check_kwargs(kwargs, _DEFAULT_KWARGS + _TL_VALUE_KWARGS)
 
+    if data is None or not isinstance(data, pd.DataFrame) or data.empty:
+        print("No data to plot.")
+        return figure()
+
     reset_output()
     output_notebook()
     height: int = kwargs.pop("height", None)
@@ -275,22 +280,24 @@ def display_timeline_values(
 
     ref_time, ref_label = _get_ref_event_time(**kwargs)
 
+    if source_columns is None:
+        source_columns = [y]
+    if y not in source_columns:
+        source_columns.append(y)
     graph_df, group_count_df, tool_tip_columns, series_count = _create_data_grouping(
         data, source_columns, time_column, group_by, color
     )
 
-    hover = HoverTool(
-        tooltips=_create_tool_tips(data, tool_tip_columns),
-        formatters={"Tooltip": "printf"},
-    )
+    tooltips, formatters = _create_tool_tips(data, tool_tip_columns)
+    hover = HoverTool(tooltips=tooltips, formatters=formatters)
 
     # Create the Plot figure
-    title = title if title else "Timeline"
+    title = title or "Timeline"
     min_time = graph_df[time_column].min()
     max_time = graph_df[time_column].max()
     start_range = min_time - ((max_time - min_time) * 0.1)
     end_range = max_time + ((max_time - min_time) * 0.1)
-    height = height if height else _calc_auto_plot_height(series_count)
+    height = height or _calc_auto_plot_height(series_count)
 
     plot = figure(
         x_range=(start_range, end_range),
@@ -465,10 +472,8 @@ def _display_timeline_dict(data: dict, **kwargs) -> figure:  # noqa: C901, MC000
     tool_tip_columns, min_time, max_time = _unpack_data_series_dict(data, **kwargs)
     series_count = len(data)
 
-    hover = HoverTool(
-        tooltips=_create_tool_tips(data, tool_tip_columns),
-        formatters={"Tooltip": "printf"},
-    )
+    tooltips, formatters = _create_tool_tips(data, tool_tip_columns)
+    hover = HoverTool(tooltips=tooltips, formatters=formatters)
 
     title = f"Timeline: {title}" if title else "Event Timeline"
     try:
@@ -552,7 +557,12 @@ def _display_timeline_dict(data: dict, **kwargs) -> figure:  # noqa: C901, MC000
                 source=series_def["source"],
             )
         if legend_pos in ["left", "right"]:
-            legend_items.append((str(ser_name), [p_series]))
+            legend_items.append(
+                LegendItem(
+                    label=str(ser_name),
+                    renderers=[p_series],
+                )
+            )
 
     if legend_pos == "inline":
         # Position the inline legend
@@ -561,7 +571,7 @@ def _display_timeline_dict(data: dict, **kwargs) -> figure:  # noqa: C901, MC000
     elif legend_pos in ["left", "right"]:
         # Create the legend box outside of the plot area
         ext_legend = Legend(
-            items=legend_items,
+            items=legend_items[::-1],  # the legend is in the wrong order otherwise
             location="center",
             click_policy="hide",
             label_text_font_size="8pt",
@@ -590,15 +600,11 @@ def _unpack_data_series_dict(data, **kwargs):
     time_column: str = kwargs.pop("time_column", "TimeGenerated")
     source_columns: list = kwargs.pop("source_columns", None)
 
-    if not source_columns:
-        source_columns = ["NewProcessName", "EventID", "CommandLine"]
-    def_source_columns = set(source_columns) if source_columns else set()
-
     # Process the input dictionary
     # Take each item that is passed and fill in blanks and add a y_index
     tool_tip_columns: Set[str] = set()
-    min_time = pd.Timestamp(pd.Timestamp.max)
-    max_time = pd.Timestamp(pd.Timestamp.min)
+    min_time = None
+    max_time = None
     y_index = 0
 
     # Create a color map in case colors have not been specified
@@ -606,25 +612,32 @@ def _unpack_data_series_dict(data, **kwargs):
     series_count = len(data)
     colors, palette_size = _get_color_palette(series_count)
 
-    for series_def in data.values():
+    for ser_name, series_def in data.items():
         data_columns: Set[str] = set()
         series_data = series_def["data"]
 
-        # if the series has source columns, use those
-        src_cols = series_def.get("source_columns", def_source_columns)
-        data_columns.update(src_cols if src_cols else def_source_columns)
+        if (
+            series_data is None
+            or not isinstance(series_data, pd.DataFrame)
+            or series_data.empty
+        ):
+            print(f"No data to plot for series {ser_name}.")
+            continue
 
-        # add these columns to the tool tip column set
-        tool_tip_columns.update(data_columns)
+        # if the series has source columns, use those
+        # or fall back to global source cols or defaults
+        series_def_src_cols = _get_def_source_cols(series_data, source_columns)
+        src_cols = series_def.get("source_columns", series_def_src_cols)
+        data_columns.update(src_cols or series_def_src_cols)
 
         time_col = series_def.get("time_column", None)
         if not time_col:
             time_col = time_column
             series_def["time_column"] = time_col
-
-        min_time = min(min_time, series_data[time_col].min())
-        max_time = max(max_time, series_data[time_col].max())
         data_columns.update([time_col])
+        # add the data columns to the tool tip column set
+        tool_tip_columns.update(data_columns)
+
         # Create the Column data source to plot
         graph_df = series_data[list(data_columns)].copy()
         graph_df["y_index"] = y_index
@@ -638,20 +651,37 @@ def _unpack_data_series_dict(data, **kwargs):
         series_def["source"] = ColumnDataSource(graph_df)
         y_index += 1
 
+        # calculate min/max time from this set
+        if min_time is None:
+            min_time = series_data[time_col].min()
+            max_time = series_data[time_col].max()
+        else:
+            min_time = min(min_time, series_data[time_col].min())
+            max_time = max(max_time, series_data[time_col].max())
+
     return tool_tip_columns, min_time, max_time
 
 
 # pylint: enable=too-many-locals
 
 
-def _create_data_grouping(data, source_columns, time_column, group_by, color):
+def _get_def_source_cols(data, source_columns):
     if not source_columns:
-        data_columns = set(["NewProcessName", "EventID", "CommandLine"])
+        data_columns = set()
+        if all(
+            col in data.columns for col in ["NewProcessName", "EventID", "CommandLine"]
+        ):
+            data_columns = set(["NewProcessName", "EventID", "CommandLine"])
     else:
         data_columns = set(source_columns)
-    tool_tip_columns = data_columns.copy()
+    return data_columns
+
+
+def _create_data_grouping(data, source_columns, time_column, group_by, color):
+    data_columns = _get_def_source_cols(data, source_columns)
     # If the time column not explicity specified in source_columns, add it
     data_columns.add(time_column)
+    tool_tip_columns = data_columns.copy()
     # create group frame so that we can color each group separately
     if group_by:
         group_count_df = (
@@ -686,10 +716,7 @@ def _create_data_grouping(data, source_columns, time_column, group_by, color):
 
 
 def _create_dict_from_grouping(data, source_columns, time_column, group_by, color):
-    if not source_columns:
-        data_columns = set(["NewProcessName", "EventID", "CommandLine"])
-    else:
-        data_columns = set(source_columns)
+    data_columns = _get_def_source_cols(data, source_columns)
     # If the time column not explicitly specified in source_columns, add it
     data_columns.add(time_column)
 
@@ -701,16 +728,13 @@ def _create_dict_from_grouping(data, source_columns, time_column, group_by, colo
 
         series_count = len(grouped_data)
         colors, palette_size = _get_color_palette(series_count)
-        color_index = 0
-        for group_name, group_df in grouped_data:
+        for color_index, (group_name, group_df) in enumerate(grouped_data):
             series_dict[str(group_name)] = dict(
                 data=group_df,
                 time_column=time_column,
                 source_columns=source_columns,
                 color=colors[color_index % palette_size],
             )
-            color_index += 1
-
     else:
         group_df = data[list(data_columns)].copy()
         series_dict["unnamed series"] = dict(
@@ -743,26 +767,47 @@ def _get_ref_event_time(**kwargs) -> Tuple[datetime, str]:
     return ref_time, kwargs.get("ref_label", ref_label)
 
 
-def _create_tool_tips(data: pd.DataFrame, columns: List[str]):
+def _get_datetime_tooltip(col: str, dataset: pd.DataFrame):
+    """Return tooltip and formatter entries for column."""
+    if " " in col:
+        disp_col = col.replace(" ", "_")
+        tt_col = f"{{{col}}}"
+    else:
+        disp_col = tt_col = col
+    if col in dataset and is_datetime64_any_dtype(dataset[col]):
+        col_tooltip = f"@{tt_col}{{%F %T.%3N}}"
+        col_fmt: Dict[Any, Any] = {f"@{tt_col}": "datetime"}
+    else:
+        col_tooltip = f"@{tt_col}"
+        col_fmt = {}
+    return disp_col, col_tooltip, col_fmt
+
+
+def _create_tool_tips(
+    data: Union[pd.DataFrame, Dict[str, pd.DataFrame]], columns: List[str]
+) -> Tuple[List[Tuple[str, str]], Dict[str, str]]:
     """Create formatting for tool tip columns."""
-    ts_dtypes = [np.dtype("<M8[ns]"), np.dtype(">M8[ns]")]
+    formatters: Dict[str, str] = {}
+    # if this is a dict we need to unpack each dataframe and process
+    # the tooltip columns for all of the data sets.
     if isinstance(data, dict):
         tool_tip_dict = {}
-        for series_df in data.values():
+        for data_set in data.values():
+            data_df = data_set.get("data", {})
             for col in columns:
-                if col in data and series_df[col].dtype in ts_dtypes:
-                    tool_tip_dict[col] = f"@{col}{{%F %T}}"
-                elif col not in tool_tip_dict:
-                    tool_tip_dict[col] = f"@{col}"
-        return list(tool_tip_dict.items())
+                disp_col, col_tooltip, col_fmt = _get_datetime_tooltip(col, data_df)
+                tool_tip_dict[disp_col] = col_tooltip
+                formatters.update(col_fmt)
+        return list(tool_tip_dict.items()), formatters
 
+    # If just a dataframe we just process the columns against this
     tool_tip_items = []
     for col in columns:
-        if col in data and data[col].dtype in ts_dtypes:
-            tool_tip_items.append((f"{col}", f"@{col}{{%F %T}}"))
-        else:
-            tool_tip_items.append((f"{col}", f"@{col}"))
-    return tool_tip_items
+        disp_col, col_tooltip, col_fmt = _get_datetime_tooltip(col, data)
+        tool_tip_items.append((disp_col, col_tooltip))
+        formatters.update(col_fmt)
+
+    return tool_tip_items, formatters
 
 
 def _get_color_palette(series_count):
@@ -929,7 +974,7 @@ def _add_ref_line(plot, ref_time, ref_text="Ref time", series_count=1):
 
 
 @pd.api.extensions.register_dataframe_accessor("mp_timeline")
-class ProcessTreeAccessor:
+class TimeLineAccessor:
     """Pandas api extension for Timeline."""
 
     def __init__(self, pandas_obj):
