@@ -4,14 +4,14 @@
 # license information.
 # --------------------------------------------------------------------------
 """Process Tree Visualization."""
-from typing import Optional, Dict, Iterable, Union, Any
+from typing import Any, Dict, Iterable, Optional, Union
 
 import attr
 import pandas as pd
 
-from ..nbtools.nbwidgets import Progress
-from ..common.utility import MsticpyException
 from .._version import VERSION
+from ..common.exceptions import MsticpyException
+from ..nbtools.nbwidgets import Progress
 
 __version__ = VERSION
 __author__ = "Ian Hellen"
@@ -208,10 +208,7 @@ def infer_schema(data: Union[pd.DataFrame, pd.Series]) -> ProcSchema:
         The schema most closely matching the data set.
 
     """
-    if isinstance(data, pd.DataFrame):
-        src_cols = data.columns
-    else:
-        src_cols = data.index
+    src_cols = data.columns if isinstance(data, pd.DataFrame) else data.index
     lx_match = set(src_cols) & set(LX_EVENT_SCH.columns)
     win_match = set(src_cols) & set(WIN_EVENT_SCH.columns)
     return LX_EVENT_SCH if len(lx_match) > len(win_match) else WIN_EVENT_SCH
@@ -299,6 +296,9 @@ def _extract_inferred_parents(
     merged_procs: pd.DataFrame, schema: ProcSchema
 ) -> pd.DataFrame:
     """Find any inferred parents and creates rows for them."""
+    tz_aware = merged_procs.iloc[0][schema.time_stamp].tz
+    time_zero = pd.Timestamp(0) if tz_aware is None else pd.Timestamp(0, tz=0)
+
     # Fill in missing values for root processes
     root_procs_crit = merged_procs["source_index_par"].isna()
     merged_procs.loc[root_procs_crit, "NewProcessId_par"] = merged_procs[
@@ -318,7 +318,7 @@ def _extract_inferred_parents(
     merged_procs.loc[root_procs_crit, "EffectiveLogonId_par"] = merged_procs[
         schema.logon_id
     ]
-    merged_procs.loc[root_procs_crit, "TimeGenerated_orig_par"] = pd.Timestamp(0)
+    merged_procs.loc[root_procs_crit, "TimeGenerated_orig_par"] = time_zero
 
     # Extract synthentic rows for the parents of root processes
     inferred_parents = (
@@ -341,17 +341,13 @@ def _extract_inferred_parents(
                 "EffectiveLogonId_par": schema.logon_id,
             }
         )
-        .assign(
-            TimeGenerated=pd.Timestamp(0),
-            EffectiveLogonId=merged_procs[schema.logon_id],
-        )
+        .assign(TimeGenerated=time_zero, EffectiveLogonId=merged_procs[schema.logon_id])
         .drop_duplicates()
     )
 
-    plus_parents = pd.concat(
+    return pd.concat(
         [merged_procs, inferred_parents], ignore_index=True, axis=0, sort=False
     )
-    return plus_parents
 
 
 def _assign_proc_keys(
@@ -401,12 +397,14 @@ def _build_proc_tree(input_tree, progress: Progress, max_depth=-1):
     input_tree["path"] = input_tree["source_index"]
 
     cur_level = input_tree[input_tree["IsRoot"]]
+    remaining_procs = input_tree[~input_tree["IsRoot"]]
 
     cur_level_num = 0
     progress.update_progress(delta=len(cur_level))
     while True:
-        sel_crit = input_tree["parent_key"].isin(cur_level.index)
-        next_level = input_tree[sel_crit].copy()
+        sel_crit = remaining_procs["parent_key"].isin(cur_level.index)
+        next_level = remaining_procs[sel_crit].copy()
+        remaining_procs = remaining_procs[~sel_crit]
 
         if next_level.empty:
             progress.update_progress(new_total=progress.max)
@@ -681,8 +679,16 @@ def get_descendents(
     descendents = []
     parent_keys = [proc.name]
     level = 0
+    rem_procs: Optional[pd.DataFrame] = None
     while max_levels == -1 or level < max_levels:
-        children = procs[procs["parent_key"].isin(parent_keys)]
+        if rem_procs is not None:
+            # pylint: disable=unsubscriptable-object
+            children = rem_procs[rem_procs["parent_key"].isin(parent_keys)]
+            rem_procs = rem_procs[~rem_procs["parent_key"].isin(parent_keys)]
+            # pylint: enable=unsubscriptable-object
+        else:
+            children = procs[procs["parent_key"].isin(parent_keys)]
+            rem_procs = procs[~procs["parent_key"].isin(parent_keys)]
         if children.empty:
             break
         descendents.append(children)
@@ -825,6 +831,7 @@ def _check_proc_keys(merged_procs_par, schema):
         merged_procs_par[schema.time_stamp]
     )
     crit2 = merged_procs_par["EffectiveLogonId"].isin(merged_procs_par[schema.logon_id])
+    c2a = None
     if schema.target_logon_id:
         c2a = merged_procs_par["EffectiveLogonId"].isin(
             merged_procs_par[schema.target_logon_id]
@@ -835,7 +842,7 @@ def _check_proc_keys(merged_procs_par, schema):
     crit6 = merged_procs_par["parent_key"].isna()
     print("has parent time", len(merged_procs_par[crit1]))
     print("effectivelogonId in subjectlogonId", len(merged_procs_par[crit2]))
-    if schema.target_logon_id:
+    if schema.target_logon_id and c2a:
         print("effectivelogonId in targetlogonId", len(merged_procs_par[c2a]))
     print("parent_proc_lc in procs", len(merged_procs_par[crit3]))
     print("ProcessId in ParentProcessId", len(merged_procs_par[crit4]))
