@@ -12,24 +12,26 @@ processing performance may be limited to a specific number of
 requests per minute for the account type that you have.
 
 """
+import sys  # noqa
+import warnings
 from collections import ChainMap
 from inspect import isclass
-import sys  # noqa
-from typing import List, Mapping, Dict, Tuple, Union, Iterable, Optional
-import warnings
+from typing import Dict, Iterable, List, Mapping, Optional, Tuple, Union
 
 import attr
 import pandas as pd
 
-# used in dynamic instantiation of providers
-# pylint: disable=unused-wildcard-import, wildcard-import
-
-from . import tiproviders
-from .tiproviders import *  # noqa:F401, F403
-from .tiproviders.ti_provider_base import TIProvider, LookupResult, TILookupStatus
+from .._version import VERSION
+from ..common.exceptions import MsticpyConfigException, MsticpyUserConfigError
 from ..common.provider_settings import get_provider_settings, reload_settings
 from ..common.utility import export
-from .._version import VERSION
+from ..nbtools.ti_browser import browse_results
+from . import tiproviders
+
+# used in dynamic instantiation of providers
+# pylint: disable=unused-wildcard-import, wildcard-import
+from .tiproviders import *  # noqa:F401, F403
+from .tiproviders.ti_provider_base import LookupResult, TILookupStatus, TIProvider
 
 __version__ = VERSION
 __author__ = "Ian Hellen"
@@ -38,9 +40,11 @@ __author__ = "Ian Hellen"
 _NO_PROVIDERS_MSSG = """
 No TI Providers are loaded - please check that
 you have correctly configured your msticpyconfig.yaml settings.
-For more information see
-https://msticpy.readthedocs.io/en/latest/TIProviders.html
 """
+_TI_HELP_URI = (
+    "https://msticpy.readthedocs.io/en/latest/data_acquisition/"
+    "TIProviders.html#configuration-file"
+)
 
 
 @export
@@ -51,6 +55,7 @@ class TILookup:
         self,
         primary_providers: Optional[List[TIProvider]] = None,
         secondary_providers: Optional[List[TIProvider]] = None,
+        providers: Optional[List[str]] = None,
     ):
         """
         Initialize TILookup instance.
@@ -61,10 +66,17 @@ class TILookup:
             Primary TI Providers, by default None
         secondary_providers : Optional[List[TIProvider]], optional
             Secondary TI Providers, by default None
+        providers: Optional[List[str]], optional
+            List of provider names to load, by default all available
+            providers are loaded. To see the list of available providers
+            call `TILookup.list_available_providers()`.
+            Note: if primary_provides or secondary_providers is specified
+            This will override the providers list.
 
         """
         self._providers: Dict[str, TIProvider] = {}
         self._secondary_providers: Dict[str, TIProvider] = {}
+        self._providers_to_load = providers
 
         if primary_providers:
             for prov in primary_providers:
@@ -72,7 +84,7 @@ class TILookup:
         if secondary_providers:
             for prov in secondary_providers:
                 self.add_provider(prov, primary=False)
-        if not primary_providers and not secondary_providers:
+        if not (primary_providers or secondary_providers):
             self._load_providers()
 
         self._all_providers = ChainMap(self._secondary_providers, self._providers)
@@ -138,10 +150,14 @@ class TILookup:
             List of TI Provider classes.
 
         """
+        return self._get_available_providers()
+
+    @classmethod
+    def _get_available_providers(cls):
         providers = []
         for provider_name in dir(tiproviders):
             provider_class = getattr(tiproviders, provider_name, None)
-            if not provider_class or not isclass(provider_class):
+            if not (provider_class and isclass(provider_class)):
                 continue
             # if it is a class - we only want to show concrete classes
             # that are sub-classes of TIProvider
@@ -151,8 +167,84 @@ class TILookup:
                 providers.append(provider_class.__name__)
         return providers
 
+    def enable_provider(self, providers: Union[str, Iterable[str]]):
+        """
+        Set the provider(s) as primary (used by default).
+
+        Parameters
+        ----------
+        providers : Union[str, Iterable[str]
+            Provider name or list of names.
+            Use `list_available_providers()` to see the list of loaded providers.
+
+        Raises
+        ------
+        ValueError
+            If the provider name is not recognized.
+
+        """
+        providers = [providers] if isinstance(providers, str) else providers
+        for provider in providers:
+            if provider in self._secondary_providers:
+                self._providers[provider] = self._secondary_providers[provider]
+                del self._secondary_providers[provider]
+            elif provider not in self._providers:
+                raise ValueError(
+                    f"Unknown provider '{provider}'. Available providers:",
+                    ", ".join(self.list_available_providers(as_list=True)),  # type: ignore
+                )
+        # disable all other providers
+        for provider in self._providers:
+            if provider not in providers:
+                self.disable_provider(provider)
+
+    def disable_provider(self, providers: Union[str, Iterable[str]]):
+        """
+        Set the provider as secondary (not used by default).
+
+        Parameters
+        ----------
+        providers : Union[str, Iterable[str]
+            Provider name or list of names.
+            Use `list_available_providers()` to see the list of loaded providers.
+
+        Raises
+        ------
+        ValueError
+            If the provider name is not recognized.
+
+        """
+        providers = [providers] if isinstance(providers, str) else providers
+        for provider in providers:
+            if provider in self._providers:
+                self._secondary_providers[provider] = self._providers[provider]
+                del self._providers[provider]
+            elif provider not in self._secondary_providers:
+                raise ValueError(
+                    f"Unknown provider '{provider}'. Available providers:",
+                    ", ".join(self.list_available_providers(as_list=True)),  # type: ignore
+                )
+
+    def set_provider_state(self, prov_dict: Dict[str, bool]):
+        """
+        Set a dict of providers to primary/secondary.
+
+        Parameters
+        ----------
+        prov_dict : Dict[str, bool]
+            Dictionary of provider name and bool - True if enabled/primary,
+            False if disabled/secondary.
+
+        """
+        for prov, state in prov_dict.items():
+            if state:
+                self.enable_provider(prov)
+            else:
+                self.disable_provider(prov)
+
+    @classmethod
     def list_available_providers(
-        self, show_query_types=False, return_list: bool = False
+        cls, show_query_types=False, as_list: bool = False
     ) -> Optional[List[str]]:  # type: ignore
         """
         Print a list of builtin providers with optional usage.
@@ -161,8 +253,10 @@ class TILookup:
         ----------
         show_query_types : bool, optional
             Show query types supported by providers, by default False
-        return_list : bool, optional
-            Return list of providers as well as printing to stdout.
+        as_list : bool, optional
+            Return list of providers instead of printing to stdout.
+            Note: if you specify `show_query_types` this will be printed
+            irrespective of this parameter setting.
 
         Returns
         -------
@@ -171,14 +265,14 @@ class TILookup:
 
         """
         providers = []
-        for provider_name in self.available_providers:
+        for provider_name in cls._get_available_providers():
             provider_class = getattr(tiproviders, provider_name, None)
-            if return_list is False:
+            if not as_list:
                 print(provider_name)
             providers.append(provider_name)
             if show_query_types:
                 provider_class.usage()
-        if return_list is True:
+        if as_list:
             return providers
         return None
 
@@ -212,7 +306,7 @@ class TILookup:
 
     def reload_providers(self):
         """
-        Reload providers based on currrent settings in config.
+        Reload providers based on current settings in config.
 
         Parameters
         ----------
@@ -229,26 +323,42 @@ class TILookup:
 
         for provider_entry, settings in prov_settings.items():
             # Allow overriding provider name to use another class
-            provider_name = settings.provider if settings.provider else provider_entry
+            provider_name = settings.provider or provider_entry
+            if self._providers_to_load and provider_name not in self._providers_to_load:
+                continue
             provider_class: TIProvider = getattr(
                 sys.modules[__name__], provider_name, None
             )
             if not provider_class:
                 warnings.warn(
                     f"Could not find provider class for {provider_name} "
-                    + f"in config section {provider_entry}"
+                    f"in config section {provider_entry}"
                 )
                 continue
 
             # instantiate class sending args from settings to init
-            provider_instance = provider_class(**(settings.args))
+            try:
+                provider_instance = provider_class(**(settings.args))
+            except MsticpyConfigException as mp_ex:
+                # If the TI Provider didn't load, raise an exception
+                raise MsticpyUserConfigError(
+                    f"Could not load TI Provider {provider_name}",
+                    *mp_ex.args,
+                    "To avoid loading this provider please use the 'providers' parameter"
+                    + " to TILookup() to specify which providers to load.",
+                    title="TIProvider configuration error",
+                    help_uri=(
+                        "https://msticpy.readthedocs.io/en/latest/data_acquisition/TIProviders.html"
+                        + "#configuration-file"
+                    ),
+                ) from mp_ex
+
             # set the description from settings, if one is provided, otherwise
             # use class docstring.
             provider_instance.description = (
-                settings.description
-                if settings.description
-                else provider_instance.__doc__
+                settings.description or provider_instance.__doc__
             )
+
             self.add_provider(
                 provider=provider_instance, name=provider_name, primary=settings.primary
             )
@@ -323,9 +433,13 @@ class TILookup:
         result_list: List[Tuple[str, LookupResult]] = []
         selected_providers = self._select_providers(providers, prov_scope)
         if not selected_providers:
-            raise RuntimeError(_NO_PROVIDERS_MSSG)
+            raise MsticpyUserConfigError(
+                _NO_PROVIDERS_MSSG,
+                title="No Threat Intel Provider configuration found.",
+                help_uri=_TI_HELP_URI,
+            )
 
-        ioc_type = ioc_type if ioc_type else TIProvider.resolve_ioc_type(observable)
+        ioc_type = ioc_type or TIProvider.resolve_ioc_type(observable)
         for prov_name, provider in selected_providers.items():
             provider_result: LookupResult = provider.lookup_ioc(
                 ioc=observable, ioc_type=ioc_type, query_type=ioc_query_type, **kwargs
@@ -357,6 +471,7 @@ class TILookup:
             3. Iterable of observables - IoCTypes will be inferred
         obs_col : str, optional
             DataFrame column to use for observables, by default None
+            ("col" and "column" are also aliases for this parameter)
         ioc_type_col : str, optional
             DataFrame column to use for IoCTypes, by default None
         ioc_query_type: str, optional
@@ -374,10 +489,16 @@ class TILookup:
             DataFrame of results
 
         """
+        obs_col = obs_col or kwargs.pop("col", kwargs.pop("column", None))
+
         result_list: List[pd.DataFrame] = []
         selected_providers = self._select_providers(providers, prov_scope)
         if not selected_providers:
-            raise RuntimeError(_NO_PROVIDERS_MSSG)
+            raise MsticpyUserConfigError(
+                _NO_PROVIDERS_MSSG,
+                title="No Threat Intel Provider configuration found.",
+                help_uri=_TI_HELP_URI,
+            )
 
         for prov_name, provider in selected_providers.items():
             provider_result = provider.lookup_iocs(
@@ -468,3 +589,35 @@ class TILookup:
             else:
                 selected_providers = self._secondary_providers
         return selected_providers
+
+    @staticmethod
+    def browse_results(
+        data: pd.DataFrame, severities: Optional[List[str]] = None, **kwargs
+    ):
+        """
+        Return TI Results list browser.
+
+        Parameters
+        ----------
+        data : pd.DataFrame
+            TI Results data from TIProviders
+        severities : Optional[List[str]], optional
+            A list of the severity classes to show.
+            By default these are ['warning', 'high'].
+            Pass ['information', 'warning', 'high'] to see all
+            results.
+
+        Other Parameters
+        ----------------
+        kwargs :
+            passed to SelectItem constuctor.
+
+        Returns
+        -------
+        SelectItem
+            SelectItem browser for TI Data.
+
+        """
+        return browse_results(data=data, severities=severities, **kwargs)
+
+    browse = browse_results
